/* 
 * i-scream central monitoring system
 * http://www.i-scream.org
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.server.client;

//---IMPORTS---
import uk.org.iscream.cms.server.componentmanager.*;
import uk.org.iscream.cms.server.core.*;
import uk.org.iscream.cms.util.*;
import java.util.LinkedList;
import java.util.StringTokenizer;

/**
 * A manager for the Monitors.
 *
 * This class starts by loading all the monitors as specificed in the configuration.
 * These monitors should implement the PluginMonitor interface.
 *
 * This class then takes the feed of data coming in over CORBA from
 * the ClientServant queue.  This data is then looked at to determine
 * type.  It then places it into either a data queue, or an other
 * queue, and all data in the the all queue.
 *
 * Monitors then read the data off the queue that they are interested in
 * and process the data accordingly.
 *
 * @author  $Author: tdb $
 * @version $Id: MonitorManager.java,v 1.22 2005/09/25 09:57:40 tdb Exp $
 */
public class MonitorManager extends Thread {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public static final String REVISION = "$Revision: 1.22 $";
    
//---STATIC METHODS---

    /**
     * Return a reference to the single class.
     * Construct it if it does not already exist, otherwise just return the reference.
     */
    public synchronized static MonitorManager getInstance() {
        if (_instance == null){
            _instance = new MonitorManager();
        }
        return _instance;
    }

//---CONSTRUCTORS---

    /**
     * Constructs a new MonitorManager.
     * This initialises all the queues and loads
     * all the Monitors that have been specified in the configuration
     */
    private MonitorManager() {
        // set our name
        setName("client.MonitorManager");
        
        _queue = ClientMain._monitorQueue;
        _qID = _queue.getQueue();
        _logger.write(toString(), Logger.SYSINIT, "Initialising");
        _logger.write(toString(), Logger.SYSMSG, "Creating monitor pipeline for plugin monitors ...");
        
        // get the config proxy
        ConfigurationProxy cp = ConfigurationProxy.getInstance();
        
        // get the configuration for our outgoing queue's
        
        // see if these Queue's need a size limit
        try {
            int queueSizeLimit = Integer.parseInt(cp.getProperty(_name, "Queue.SizeLimit"));
            String queueRemoveAlgorithm = cp.getProperty(_name, "Queue.RemoveAlgorithm");
            int algorithm = StringUtils.getStringPos(queueRemoveAlgorithm, Queue.algorithms);
            if(algorithm != -1) {
                _logger.write(toString(), Logger.DEBUG, "Starting 3 Queues with size limit of "+queueSizeLimit+", using remove algorithm "+queueRemoveAlgorithm);
                // we have valid values, so lets start it.
                _dataQueue = new Queue(queueSizeLimit, algorithm);
                _otherQueue = new Queue(queueSizeLimit, algorithm);
                _allQueue = new Queue(queueSizeLimit, algorithm);
            }
            else {
                _logger.write(toString(), Logger.WARNING, "Bad Queue Algorithm configuration, not known: "+queueRemoveAlgorithm);
                // just don't activate a limit
                _dataQueue = new Queue();
                _otherQueue = new Queue();
                _allQueue = new Queue();
            }
        } catch (PropertyNotFoundException e) {
            _logger.write(toString(), Logger.DEBUG, "Optional config not set: "+e);
            // just don't activate a limit
            _dataQueue = new Queue();
            _otherQueue = new Queue();
            _allQueue = new Queue();
        } catch (NumberFormatException e) {
            _logger.write(toString(), Logger.WARNING, "Bad Queue SizeLimit configuration: "+e);
            // just don't activate a limit
            _dataQueue = new Queue();
            _otherQueue = new Queue();
            _allQueue = new Queue();
        }
        
        // startup monitors on these queues
        try {
            // try to get the interval, if this fails, we won't start up the monitor
            int queueMonitorInterval = Integer.parseInt(cp.getProperty(_name, "Queue.MonitorInterval"));
            _dataQueue.startMonitor(queueMonitorInterval*1000, ClientMain._monitorQueue, _name + " DataQueue");
            _otherQueue.startMonitor(queueMonitorInterval*1000, ClientMain._monitorQueue, _name + " OtherQueue");
            _allQueue.startMonitor(queueMonitorInterval*1000, ClientMain._monitorQueue, _name + " AllQueue");
        } catch (PropertyNotFoundException e) {
            _logger.write(toString(), Logger.WARNING, "failed to find queue monitor config, disabling. " + e);
        }
        
        // get the configuration for this plug-in setup
        String pluginsPackage, pluginsList;
        try {
            pluginsPackage = cp.getProperty(_name, "Monitor.PluginsPackage");
            pluginsList = cp.getProperty(_name, "Monitor.Plugins");
        } catch(PropertyNotFoundException e) {
            _logger.write(toString(), Logger.WARNING, "Unable to get required configuration, Monitor's will not be activated: "+e);
            // setting these will ensure we don't build a pipeline
            pluginsPackage = "";
            pluginsList = "";
        }
        
        StringTokenizer st = new StringTokenizer(pluginsList, ";");
        
        while(st.hasMoreTokens()) {
            String className = pluginsPackage + "." + st.nextToken() + _suffix;
            _logger.write(toString(), Logger.DEBUG, "Attempting to create plugin: "+className);
            
            // Create an instance of the specified PluginMonitor to include
            // within the monitorPipe.  Add it to the monitorPipeline
            try {
                PluginMonitor pm = (PluginMonitor)ClassLoader.getSystemClassLoader().loadClass(className).newInstance();
                _monitorPipeline.add(pm);
                _logger.write(toString(), Logger.DEBUG, "Added monitor: "+className+" ("+pm.getDescription()+")");
            }
            catch (InstantiationException e){
                _logger.write(toString(), Logger.ERROR, "Failed to instantiate "+className+" to the plugin monitor pipeline.");
                _logger.write(toString(), Logger.ERROR, e.getMessage());
            }
            catch (Exception e){
                _logger.write(toString(), Logger.ERROR, "Failed to add "+className+" to the plugin monitor pipeline.");
                _logger.write(toString(), Logger.ERROR, e.toString());
            }
        }
        _logger.write(toString(), Logger.SYSMSG, "The monitor pipeline has been set up with "+_monitorPipeline.size()+" plugin monitors.");
    }

//---PUBLIC METHODS---
    
    /**
     * Runs the MonitorManager.  This reads data from the
     * inbound queue (from the ClientServant).  And passes
     * it into the appropriate queue for processing by the monitor
     * threads.
     */
    public void run() {
        boolean run=true;
        
        // keep these out here, saves recreating the object
        String xml = null;
        while(run) {
            try {
                xml = (String) _queue.get(_qID);
            }
            catch(InvalidQueueException e) {
                _logger.write(toString(), Logger.ERROR, "Queue failure: "+e);
            }
            
            // make an XML packet
            XMLPacket packet = null;
            
            try {
                packet = _xmlCache.getXMLPacket(xml);
            } catch(InvalidXMLException e) {
                _logger.write(toString(), Logger.ERROR, "Invalid XML: "+e);
                // skip the rest of this loop iteration
                continue;
            }

            if(packet == null) {
                _logger.write(toString(), Logger.WARNING, "Got a null packet when parsing: "+xml);
                // skip to next packet
                continue;
            }
            
            // examine the packet and place it in the relevant outgoing queue
            if(packet.getParam("packet.attributes.type") != null &&
	       packet.getParam("packet.attributes.type").equals("data")) {
                _dataQueue.add(packet);
            }
            else {
                _otherQueue.add(packet);
            }
            // always add to all queue
            _allQueue.add(packet);
        }
    }

    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.util.FormatName class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            _name,
            getClass().getName(),
            REVISION);
    }

//---PRIVATE METHODS---

//---ACCESSOR/MUTATOR METHODS---
    
    /**
     * Allows Monitors to obtain
     * the queue of data packets
     */
    public Queue getDataQueue() {
        return _dataQueue;
    }
    
    /**
     * Allows Monitors to obtain
     * the queue of all other packets
     */
    public Queue getOtherQueue() {
        return _otherQueue;
    }
    
    /**
     * In case a Monitor wants more
     * than one type of packet,
     * this queue can be obtained.
     */
    public Queue getAllQueue() {
        return _allQueue;
    }
    
//---ATTRIBUTES---

    /**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    private String _name = ClientMain.NAME;

    /**
     * This holds a reference to the
     * system logger that is being used.
     */
    private Logger _logger = ReferenceManager.getInstance().getLogger();
    
    /**
     * A reference to the reference manager in use
     */
    private ReferenceManager _refman = ReferenceManager.getInstance();
    
    /**
     * A reference to our incoming Queue
     */
    private Queue _queue;
    
    /**
     * Our incoming queue ID
     */
    private int _qID;
    
    /**
     * file name suffix for plugin monitor classes:
     */
    private final String _suffix = "__Monitor";
    
    /**
     * LinkedList for holding the PluginMonitor objects (the pipeline).
     */
    private LinkedList _monitorPipeline = new LinkedList();
    
    /**
     * Outgoing data Queue
     */
    private Queue _dataQueue;
    
    /**
     * Outgoing other Queue
     */
    private Queue _otherQueue;
    
    /**
     * Outgoing ALL Queue
     */
    private Queue _allQueue;
    
    /**
     * A reference to the XMLCache in use
     */
    private XMLCache _xmlCache = XMLCache.getInstance();
    
//---STATIC ATTRIBUTES---

    /**
     * A reference to the single instance of this class
     */
    private static MonitorManager _instance;

}
