/* 
 * i-scream central monitoring system
 * http://www.i-scream.org
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.server.client.monitors;

//---IMPORTS---
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;
import java.util.Iterator;
import uk.org.iscream.cms.server.client.*;
import uk.org.iscream.cms.server.core.*;
import uk.org.iscream.cms.util.*;
import uk.org.iscream.cms.server.componentmanager.*;

/**
 * This Monitor watches the mail queues on all machines
 *
 * @author  $Author: tdb $
 * @version $Id: MailQ__Monitor.java,v 1.1 2006/11/27 10:51:54 tdb Exp $
 */
public class MailQ__Monitor extends MonitorSkeleton {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.1 $";
    
    /**
     * A description of this monitor
     */
    public final String DESC = "Monitors mail queue on all hosts.";
    
//---STATIC METHODS---

//---CONSTRUCTORS---

//---PUBLIC METHODS---
    
    /**
     * Analyse a packet of data, and generate an alert if
     * necessary.
     *
     * @param packet the XMLPacket to analyse
     */
    public void analysePacket(XMLPacket packet) {
        String source = packet.getParam("packet.attributes.machine_name");
        if(!checkBooleanConfig("Host." + source, "Monitor." + _name + ".enable")) {
            return;
        } 
        if (!_hosts.containsKey(source)) {
            _hosts.put(source, new HashMap());
        }

        HashMap mailqRegisters = (HashMap) _hosts.get(source);
        
        // key prefix
        String keyPrefix = "packet.mailq.p";
        
        // a temporary holder for all the mail queues we find
        HashSet mailqueues = new HashSet();

        // unfortunatly we need to check the whole packet to find
        // the interfaces, and then get the data attributes
        Set packetSet = packet.getSet();
        Iterator i = packetSet.iterator();
        while (i.hasNext()) {
            String dataKey = (String) i.next();
            if(dataKey.startsWith(keyPrefix)) {
                // pos is after "packet.mailq.p"
                int pos = keyPrefix.length();
                String qNumber = dataKey.substring(pos, dataKey.indexOf('.', pos));
                String thisQ = keyPrefix.concat(qNumber);
                
                if(!mailqueues.contains(thisQ)) {
                    // add the queue to our list
                    mailqueues.add(thisQ);

                    String name = packet.getParam(thisQ + ".attributes.name");
                    
                    String size = packet.getParam(thisQ + ".attributes.size");

                    // check if we've seen this queue before on a previous
                    // run if not, we need to create a register for it
                    if(!mailqRegisters.containsKey(name)) {
                        mailqRegisters.put(name, new Register(source, _name + ".size", name));
                    }

                    // get the register for this queue
                    Register reg = (Register) mailqRegisters.get(name);

                    // get the packet data
                    int queueSize;
                    try {
                        if(size==null) {
                            throw new NumberFormatException("Mail queue data invalid");
                        }
                        queueSize = Integer.parseInt(size);
                    } catch (NumberFormatException e) {
                        _logger.write(this.toString(), Logger.WARNING, "Received packet from "+source+" with bad mail queue information: "+e);
                        // don't try to continue and process, try next queue
                        break;
                    }
                    
                    int newThreshold = checkAttributeThreshold(queueSize, reg);

                    // say which queue had the problem
                    String attributeName = "Mail Queue size on " + name;

                    processAlert(newThreshold, attributeName, reg, source, String.valueOf(queueSize));
                }
            }
        }
    }

    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.util.NameFormat class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            _name,
            getClass().getName(),
            REVISION);
    }

    /**
     * return the String representation of what the monitor does
     */
    public String getDescription(){
        return DESC;
    }

//---PRIVATE METHODS---
    
    /**
     * Checks a piece of current data, and returns the
     * threshold it breaches, if any.
     *
     * @param value the current value
     * @param reg the Register for the host
     * @return the threshold level breached, if any
     */
    private int checkAttributeThreshold(double value, Register reg) {
        for(int thresholdLevel = Alert.thresholdLevels.length - 1; thresholdLevel >= 0; thresholdLevel--) {
            if (reg.getThreshold(thresholdLevel) != -1.0) {
                // normal check - has the value gone *over* the threshold
                if(((double) reg.getThreshold(thresholdLevel)) < value) {
                    return thresholdLevel;
                }
            }
        }
        return Alert.thresholdNORMAL;
    }

//---ACCESSOR/MUTATOR METHODS---
    
    /**
     * Returns a reference to a specific Queue for this
     * monitor. This Queue returns only the data packets
     * (based on type) that we want too look at.
     *
     * @return a reference to a Queue
     */
    protected Queue getQueue() {
        return MonitorManager.getInstance().getDataQueue();
    }
    
//---ATTRIBUTES---

    /**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    private String _name = "MailQ";
    
    /**
     * A reference to the configuration proxy in use
     */
    private ConfigurationProxy _cp = ConfigurationProxy.getInstance();
    
    /**
     * A HashMap of Registers (or groups of Registers), one
     * for each host we're monitoring (reads).
     */
    private HashMap _hosts = new HashMap();
    
//---STATIC ATTRIBUTES---

}
