/* 
 * i-scream central monitoring system
 * http://www.i-scream.org
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.server.filtermanager;

//---IMPORTS---
import uk.org.iscream.cms.server.core.*;
import uk.org.iscream.cms.server.filter.*;
import uk.org.iscream.cms.util.*;
import uk.org.iscream.cms.server.componentmanager.*;
import java.net.*;
import java.io.*;
import java.util.*;

/**
 * Handles setting up a host.
 * This class provides a host with appropriate configuration
 * and a reference to a Filter to which it should pass data.
 *
 * @author  $Author: tdb $
 * @version $Id: HostInit.java,v 1.41 2005/09/25 09:57:42 tdb Exp $
 */
class HostInit extends Thread {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.41 $";
    
//---STATIC METHODS---

//---CONSTRUCTORS---

    /**
     * Construct a new HostInit.
     *
     * @param socket The socket to which the host is connected
     */
    public HostInit(Socket socket) throws IOException {
        // set the Thread name
        setName("filtermanager.HostInit");
        
        _socket = socket;
        // setup reader & writer
        _socketIn = new BufferedReader(new InputStreamReader(_socket.getInputStream()));
        _socketOut = new PrintWriter(_socket.getOutputStream(), true);
        _logger.write(toString(), Logger.SYSINIT, "created");
    }
    
//---PUBLIC METHODS---

    /**
     * Main method in this class, which handles communicating with
     * the host to determine it's setup.
     */
    public void run() {
        // get a hook on the config proxy
        ConfigurationProxy cp = ConfigurationProxy.getInstance();
        // this is our config name
        String configName = "Host." + _socket.getInetAddress().getHostName().toLowerCase();
        try {
            // look for a command:
            // STARTCONFIG - to get a configuration
            // CHECKCONFIG - to check a configuration
            // END - to finish
            String cmd = getInBound();
            while(!cmd.equals("END")) {
                if(cmd.equals("STARTCONFIG")) {
                    // respond to STARTCONFIG
                    send("OK");
                    
                    // try for LASTMODIFIED
                    getInBound("LASTMODIFIED");
                    send("" + cp.getLastModified(configName));
                    
                    // try for FILELIST
                    getInBound("FILELIST");
                    send(cp.getFileList(configName));
                    
                    // try for FQDN
                    getInBound("FQDN");
                    send(_socket.getInetAddress().getHostName().toLowerCase());
                    
                    // try for IP
                    getInBound("IP");
                    send(_socket.getInetAddress().getHostAddress());
                    
                    // get properties
                    String reqProperty = getInBound();
                    while(!reqProperty.equals("ENDCONFIG")) {
                        // get the property
                        try {
                            String returnedProperty = cp.getProperty(configName, "Host."+reqProperty);
                            send(returnedProperty);
                        } catch (PropertyNotFoundException e) {
                            send("ERROR");
                        }
                        // get the next request
                        reqProperty = _socketIn.readLine();
                    }
                    _logger.write(toString(), Logger.SYSMSG, "configured host");
                    send("OK");
                    
                    // get filter reference
                    getInBound("FILTER");
                    // send info
                    String filterList = cp.getProperty(configName, "Host.filter");
                    FilterInfo filterInfoRef = null;
                    String filter = null;
                    StringTokenizer st = new StringTokenizer(filterList, ";");
                    while (filterInfoRef==null && st.hasMoreTokens()) {
                        filter = st.nextToken();
                        _logger.write(toString(), Logger.DEBUG, " looking for filter- " + filter);
                        try {
                            filterInfoRef = FilterInfoHelper.narrow(ReferenceManager.getInstance().getCORBARef("iscream.FilterInfo\\." + filter));
                        } catch (ComponentCORBAException e) {
                            _logger.write(toString(), Logger.DEBUG, " unable to find filter- " + filter);
                        }
                    }
                    
                    // hopefully we found a filter
                    if(filterInfoRef != null) {
                        _logger.write(toString(), Logger.DEBUG, " found filter- " + filter);
                        try {
                            // tell the host about it...
                            send(filterInfoRef.getHostName() + ";"
                            + filterInfoRef.getUDPPort());
                        }
                        catch(org.omg.CORBA.COMM_FAILURE e) {
                            // failed to talk to filter, lets signal an error
                            send("ERROR");
                            throw new IOException("error communicating with filter - " + e);
                        }
                    }
                    else {
                        // ...or throw a wobbly (and tell the host!)
                        send("ERROR");
                        throw new IOException("unable to find filter for host");
                    }
                    
                }
                else if(cmd.equals("CHECKCONFIG")) {
                    // respond to CHECKCONFIG
                    send("OK");
                    
                    // try for {filelist}
                    String filelist = getInBound();
                    send("OK");
                    
                    // try for {lastModified}
                    String lastModified = getInBound();
                    long lastmod;
                    try {
                        lastmod = Long.parseLong(lastModified);
                    }
                    catch(NumberFormatException e) {
                        send("ERROR");
                        throw new IOException("Last Modified invalid: " + e);
                    }
                    
                    // check to see if a config update has happen
                    boolean newConfig = _configManager.isModified(filelist, lastmod);
                    if(newConfig) {
                        // new config !
                        send("EXPIRED");
                    }
                    else {
                        // nothing has changed
                        send("OK");
                    }
                }
                else {
                    send("ERROR");
                }
                // get the next command
                cmd = getInBound();
            }
            // respond to END
            send("OK");
            
        } catch (Exception e) {
            _logger.write(toString(), Logger.ERROR, "ERROR - " + e);
        }
        
        // Disconnect streams & socket
        try {
            _socketIn.close();
            _socketOut.close();
            _socket.close();
        } catch (IOException e) {
            _logger.write(toString(), Logger.ERROR, "exception on socket close");
        }
        _logger.write(toString(), Logger.SYSMSG, "finished");
    }
    
    /**
     * Overrides the {@link java.lang.Object#toString() Object.toString()}
     * method to provide clean logging (every class should have this).
     *
     * This uses the uk.org.iscream.cms.util.NameFormat class
     * to format the toString()
     *
     * @return the name of this class and its CVS revision
     */
    public String toString() {
        return FormatName.getName(
            _name,
            getClass().getName(),
            REVISION);
    }

//---PRIVATE METHODS---

    private String getInBound(String expected) throws IOException {
        // grab the input
        String inBound = getInBound();
        // check if it's what we're expecting
        if(!inBound.equals(expected)) {
            send("ERROR");
            throw new IOException("protocol error - expected:"+expected+" got:" + inBound);
        }
        // it should be ok then
        return inBound;
    }
    
    private String getInBound() throws IOException {
        // grab the input
        String inBound = _socketIn.readLine();
        // check for null's, likely disconnection
        if(inBound == null) {
            throw new IOException("got null from host, maybe it died");
        }
        // it's a valid message it seems
        return inBound;
    }

	/* Use this instead of println. println under windows terminates the lines
	 * in \c\n which is not the expected result for the ihost client.
	 */
	private void send(String data) {
		_socketOut.print(data + "\n");
		_socketOut.flush();
	}

//---ACCESSOR/MUTATOR METHODS---

//---ATTRIBUTES---

    /**
     * This holds a reference to the
     * system logger that is being used.
     */
    private Logger _logger = ReferenceManager.getInstance().getLogger();
    
    /**
     * A reference to the Configuration Manager the system is using
     */
    private ConfigurationManager _configManager = ReferenceManager.getInstance().getCM();
    
    /**
     * This is the friendly identifier of the
     * component this class is running in.
     * eg, a Filter may be called "filter1",
     * If this class does not have an owning
     * component,  a name from the configuration
     * can be placed here.  This name could also
     * be changed to null for utility classes.
     */
    private String _name = FilterManager.NAME;
    
    /** 
     * The socket this class uses
     */
    private Socket _socket;

    /**
     * Used for the input stream of this socket
     */
    private BufferedReader _socketIn;
    
    /**
     * Used for the output stream of this socket
     */
    private PrintWriter _socketOut;

//---STATIC ATTRIBUTES---

}
